@@ -9,31 +9,32 @@ from unittest.mock import MagicMock
 
 from wtfis.clients.ipwhois import IpWhoisClient
 from wtfis.clients.shodan import ShodanClient
+from wtfis.models.ip2whois import Whois as Ip2Whois
 from wtfis.models.ipwhois import IpWhoisMap
-from wtfis.models.passivetotal import Whois
+from wtfis.models.passivetotal import Whois as PTWhois
 from wtfis.models.virustotal import (
     Domain,
-    HistoricalWhois,
     Resolutions,
+    Whois as VTWhois,
 )
 from wtfis.ui.view import DomainView
 
 
 @pytest.fixture()
 def view01(test_data, mock_ipwhois_get):
-    """ gist.github.com with PT whois. Complete test of all panels. """
+    """ gist.github.com with PT whois. Complete test of all panels. Also test print(). """
     resolutions = Resolutions.parse_obj(json.loads(test_data("vt_resolutions_gist.json")))
 
     ipwhois_pool = json.loads(test_data("ipwhois_gist.json"))
     ipwhois_client = IpWhoisClient()
     ipwhois_client.get_ipwhois = MagicMock(side_effect=lambda ip: mock_ipwhois_get(ip, ipwhois_pool))
-    ip_enrich = ipwhois_client.bulk_get_ipwhois(resolutions, 3)
+    ip_enrich = ipwhois_client.bulk_get_ip(resolutions, 3)
 
     return DomainView(
         console=Console(),
         entity=Domain.parse_obj(json.loads(test_data("vt_domain_gist.json"))),
         resolutions=resolutions,
-        whois=Whois.parse_obj(json.loads(test_data("pt_whois_gist.json"))),
+        whois=PTWhois.parse_obj(json.loads(test_data("pt_whois_gist.json"))),
         ip_enrich=ip_enrich,
     )
 
@@ -48,7 +49,7 @@ def view02(test_data):
         console=Console(),
         entity=MagicMock(),
         resolutions=Resolutions.parse_obj(json.loads(test_data("vt_resolutions_gist.json"))),
-        whois=HistoricalWhois.parse_obj(json.loads(test_data("vt_whois_gist.json"))),
+        whois=VTWhois.parse_obj(json.loads(test_data("vt_whois_gist.json"))),
         ip_enrich=IpWhoisMap(__root__={}),
         max_resolutions=1,
     )
@@ -61,7 +62,7 @@ def view03(test_data):
         console=Console(),
         entity=MagicMock(),
         resolutions=MagicMock(),
-        whois=HistoricalWhois.parse_obj(json.loads(test_data("vt_whois_bbc.json"))),
+        whois=VTWhois.parse_obj(json.loads(test_data("vt_whois_bbc.json"))),
         ip_enrich=MagicMock(),
     )
 
@@ -100,7 +101,7 @@ def view06(test_data):
         console=Console(),
         entity=MagicMock(),
         resolutions=MagicMock(),
-        whois=HistoricalWhois.parse_obj(json.loads(test_data("vt_whois_example_2.json"))),
+        whois=VTWhois.parse_obj(json.loads(test_data("vt_whois_example_2.json"))),
         ip_enrich=MagicMock(),
     )
 
@@ -164,8 +165,63 @@ def view09(test_data, mock_shodan_get_ip):
     )
 
 
+@pytest.fixture()
+def view10(test_data):
+    """ Dummy VT whois. Whois panel test only. Test whois with no data. """
+    return DomainView(
+        console=Console(),
+        entity=MagicMock(),
+        resolutions=MagicMock(),
+        whois=VTWhois.parse_obj(json.loads(test_data("vt_whois_foo.json"))),
+        ip_enrich=MagicMock(),
+    )
+
+
+@pytest.fixture()
+def view11(test_data, mock_shodan_get_ip):
+    """ gist.github.com with Shodan. Only test IP enrich. Test empty open ports. """
+    resolutions = Resolutions.parse_obj(json.loads(test_data("vt_resolutions_gist.json")))
+
+    shodan_pool = json.loads(test_data("shodan_gist_2.json"))
+    shodan_client = ShodanClient(MagicMock())
+    shodan_client.get_ip = MagicMock(side_effect=lambda ip: mock_shodan_get_ip(ip, shodan_pool))
+    ip_enrich = shodan_client.bulk_get_ip(resolutions, 3)
+
+    return DomainView(
+        console=Console(),
+        entity=MagicMock(),
+        resolutions=resolutions,
+        whois=MagicMock(),
+        ip_enrich=ip_enrich,
+    )
+
+
+@pytest.fixture()
+def view12(test_data):
+    """ Dummy IP2WHOIS whois. Whois panel test only. """
+    return DomainView(
+        console=Console(),
+        entity=MagicMock(),
+        resolutions=MagicMock(),
+        whois=Ip2Whois.parse_obj(json.loads(test_data("ip2whois_whois_hotmail.json"))),
+        ip_enrich=MagicMock(),
+    )
+
+
+@pytest.fixture()
+def view13(test_data):
+    """ Dummy IP2WHOIS whois. Whois panel test only. Test null registrant. """
+    return DomainView(
+        console=Console(),
+        entity=MagicMock(),
+        resolutions=MagicMock(),
+        whois=Ip2Whois.parse_obj(json.loads(test_data("ip2whois_whois_bbc.json"))),
+        ip_enrich=MagicMock(),
+    )
+
+
 class TestView01:
-    def test_domain_panel(self, view01):
+    def test_domain_panel(self, view01, display_timestamp):
         domain = view01.domain_panel()
         assert type(domain) is Panel
         assert domain.title == Text("virustotal")
@@ -185,7 +241,8 @@ class TestView01:
             "Analysis:",
             "Reputation:",
             "Popularity:",
-            "Last Modified:",
+            "Categories:",
+            "Updated:",
             "Last Seen:",
         ]
         assert table.columns[1].style == "none"
@@ -202,11 +259,25 @@ class TestView01:
                     Span(31, 36, "cyan"),
                 ]
             ),
-            "2022-08-16T06:14:59Z",
-            "2022-08-15T22:25:30Z",
+            Text(
+                "advice, file sharing/storage, information technology, media sharing, social networks",
+                spans=[
+                    Span(0, 6, "bright_white on black"),
+                    Span(6, 8, "default"),
+                    Span(8, 28, "bright_white on black"),
+                    Span(28, 30, "default"),
+                    Span(30, 52, "bright_white on black"),
+                    Span(52, 54, "default"),
+                    Span(54, 67, "bright_white on black"),
+                    Span(67, 69, "default"),
+                    Span(69, 84, "bright_white on black"),
+                ]
+            ),
+            display_timestamp("2022-08-16T06:14:59Z"),
+            display_timestamp("2022-08-15T22:25:30Z"),
         ]
 
-    def test_resolutions_panel(self, view01):
+    def test_resolutions_panel(self, view01, theme, display_timestamp):
         res = view01.resolutions_panel()
         assert type(res) is Panel
 
@@ -233,8 +304,11 @@ class TestView01:
         assert group[1].columns[1].justify == "left"
         assert group[1].columns[1]._cells == [
             Text("0/94 malicious"),
-            "2022-08-06T14:56:20Z",
-            "16509 (Amazon Data Services India)",
+            display_timestamp("2022-08-06T14:56:20Z"),
+            Text(
+                "16509 (Amazon Data Services India)",
+                spans=[Span(7, 33, theme.asn_org)]
+            ),
             "Amazon.com, Inc.",
             Text(
                 "Mumbai, Maharashtra, India",
@@ -271,8 +345,11 @@ class TestView01:
         assert group[1].columns[1].justify == "left"
         assert group[1].columns[1]._cells == [
             Text("1/94 malicious"),
-            "2022-06-21T18:10:54Z",
-            "36459 (GitHub, Inc.)",
+            display_timestamp("2022-06-21T18:10:54Z"),
+            Text(
+                "36459 (GitHub, Inc.)",
+                spans=[Span(7, 19, theme.asn_org)]
+            ),
             "GitHub, Inc.",
             Text(
                 "Seattle, Washington, United States",
@@ -312,8 +389,11 @@ class TestView01:
         assert table.columns[1].justify == "left"
         assert table.columns[1]._cells == [
             Text("0/94 malicious"),
-            "2015-08-17T07:11:53Z",
-            "16509 (Amazon Data Services India)",
+            display_timestamp("2015-08-17T07:11:53Z"),
+            Text(
+                "16509 (Amazon Data Services India)",
+                spans=[Span(7, 33, theme.asn_org)]
+            ),
             "Amazon.com, Inc.",
             Text(
                 "Mumbai, Maharashtra, India",
@@ -330,7 +410,7 @@ class TestView01:
         # Spacing and remaining count
         assert res.renderable.renderables[5] == Text("\n+34 more")
 
-    def test_whois_panel(self, view01):
+    def test_whois_panel(self, view01, theme, display_timestamp):
         whois = view01.whois_panel()
         assert type(whois) is Panel
         assert whois.title == Text("whois")
@@ -351,8 +431,12 @@ class TestView01:
             "Organization:",
             "Name:",
             "Email:",
+            "Phone:",
+            "Street:",
+            "City:",
             "State:",
             "Country:",
+            "Postcode:",
             "Nameservers:",
             "Registered:",
             "Updated:",
@@ -360,23 +444,61 @@ class TestView01:
         ]
         assert table.columns[1].style == "none"
         assert table.columns[1].justify == "left"
-        assert [str(c) for c in table.columns[1]._cells] == [
+        assert table.columns[1]._cells == [
             "MarkMonitor Inc.",
-            "GitHub, Inc.",
+            Text(
+                "GitHub, Inc.",
+                spans=[]
+            ),
             "N/A",
             "abusecomplaints@markmonitor.com",
+            "+1.5555555",
+            "742 Evergreen Terrace",
+            "Gotham",
             "CA",
             "US",
-            ("dns1.p08.nsone.net, dns2.p08.nsone.net, dns3.p08.nsone.net, dns4.p08.nsone.net, ns-1283.awsdns-32.org, "
-             "ns-1707.awsdns-21.co.uk, ns-421.awsdns-52.com, ns-520.awsdns-01.net"),
-            "2007-10-09T18:20:50Z",
-            "2020-09-08T09:18:27Z",
-            "2022-10-09T07:00:00Z"
+            "00000",
+            Text(
+                ("dns1.p08.nsone.net, dns2.p08.nsone.net, dns3.p08.nsone.net, dns4.p08.nsone.net, "
+                 "ns-1283.awsdns-32.org, ns-1707.awsdns-21.co.uk, ns-421.awsdns-52.com, ns-520.awsdns-01.net"),
+                spans=[
+                    Span(0, 18, theme.nameserver_list),
+                    Span(18, 20, "default"),
+                    Span(20, 38, theme.nameserver_list),
+                    Span(38, 40, "default"),
+                    Span(40, 58, theme.nameserver_list),
+                    Span(58, 60, "default"),
+                    Span(60, 78, theme.nameserver_list),
+                    Span(78, 80, "default"),
+                    Span(80, 101, theme.nameserver_list),
+                    Span(101, 103, "default"),
+                    Span(103, 126, theme.nameserver_list),
+                    Span(126, 128, "default"),
+                    Span(128, 148, theme.nameserver_list),
+                    Span(148, 150, "default"),
+                    Span(150, 170, theme.nameserver_list)
+                ]
+            ),
+            display_timestamp("2007-10-09T18:20:50Z"),
+            display_timestamp("2020-09-08T09:18:27Z"),
+            display_timestamp("2022-10-09T07:00:00Z"),
         ]
 
+    def test_print(self, view01):
+        view01.domain_panel = MagicMock()
+        view01.resolutions_panel = MagicMock()
+        view01.whois_panel = MagicMock()
+        view01.console.print = MagicMock()
+
+        view01.print()
+        view01.domain_panel.assert_called_once()
+        view01.resolutions_panel.assert_called_once()
+        view01.whois_panel.assert_called_once()
+        view01.console.print.assert_called_once()
+
 
 class TestView02:
-    def test_resolutions_panel(self, view02):
+    def test_resolutions_panel(self, view02, display_timestamp):
         res = view02.resolutions_panel()
         assert type(res) is Panel
 
@@ -400,20 +522,20 @@ class TestView02:
         assert group[1].columns[1].justify == "left"
         assert group[1].columns[1]._cells == [
             Text("0/94 malicious"),
-            "2022-08-06T14:56:20Z",
+            display_timestamp("2022-08-06T14:56:20Z"),
         ]
 
         # Spacing and remaining count
         assert res.renderable.renderables[1] == Text("\n+36 more")
 
-    def test_whois_panel(self, view02):
+    def test_whois_panel(self, view02, display_timestamp):
         whois = view02.whois_panel()
         assert type(whois) is Panel
         assert whois.title == Text("whois")
 
         # Heading
         assert whois.renderable.renderables[0] == Text(
-            "GITHUB.COM",
+            "github.com",
             spans=[Span(0, 10, 'bold yellow')]
         )
 
@@ -425,17 +547,19 @@ class TestView02:
         assert table.columns[0]._cells == [
             "Registrar:",
             "Nameservers:",
+            "DNSSEC:",
             "Registered:",
             "Updated:",
             "Expires:",
         ]
         assert table.columns[1].style == "none"
         assert table.columns[1].justify == "left"
-        assert [str(c) for c in table.columns[1]._cells] == [
+        assert [str(c) for c in table.columns[1]._cells] == [  # Just test the strings (no style)
             "MarkMonitor Inc.",
-            "DNS1.P08.NSONE.NET, DNS2.P08.NSONE.NET, DNS3.P08.NSONE.NET, "
-            "DNS4.P08.NSONE.NET, NS-1283.AWSDNS-32.ORG, NS-1707.AWSDNS-21.CO.UK, "
-            "NS-421.AWSDNS-52.COM, NS-520.AWSDNS-01.NET",
+            "dns1.p08.nsone.net, dns2.p08.nsone.net, dns3.p08.nsone.net, "
+            "dns4.p08.nsone.net, ns-1283.awsdns-32.org, ns-1707.awsdns-21.co.uk, "
+            "ns-421.awsdns-52.com, ns-520.awsdns-01.net",
+            "unsigned",
             "2007-10-09T18:20:50Z",
             "2020-09-08T09:18:27Z",
             "2022-10-09T18:20:50Z",
@@ -472,7 +596,7 @@ class TestView03:
 
 
 class TestView04:
-    def test_domain_panel(self, view04):
+    def test_domain_panel(self, view04, display_timestamp):
         domain = view04.domain_panel()
         assert type(domain) is Panel
         assert domain.title == Text("virustotal")
@@ -492,7 +616,8 @@ class TestView04:
             "Analysis:",
             "Reputation:",
             "Popularity:",
-            "Last Modified:",
+            "Categories:",
+            "Updated:",
             "Last Seen:",
         ]
         assert table.columns[1].style == "none"
@@ -521,8 +646,22 @@ class TestView04:
                     Span(65, 66, "cyan"),
                 ]
             ),
-            "2022-08-17T06:03:03Z",
-            "2022-08-17T00:35:19Z",
+            Text(
+                "mobile communications, portals, search engines, search engines and portals, searchengines",
+                spans=[
+                    Span(0, 21, "bright_white on black"),
+                    Span(21, 23, "default"),
+                    Span(23, 30, "bright_white on black"),
+                    Span(30, 32, "default"),
+                    Span(32, 46, "bright_white on black"),
+                    Span(46, 48, "default"),
+                    Span(48, 74, "bright_white on black"),
+                    Span(74, 76, "default"),
+                    Span(76, 89, "bright_white on black"),
+                ]
+            ),
+            display_timestamp("2022-08-17T06:03:03Z"),
+            display_timestamp("2022-08-17T00:35:19Z"),
         ]
 
     def test_resolutions_panel(self, view04):
@@ -531,7 +670,7 @@ class TestView04:
 
 
 class TestView05:
-    def test_domain_panel(self, view05):
+    def test_domain_panel(self, view05, display_timestamp):
         domain = view05.domain_panel()
         assert type(domain) is Panel
         assert domain.title == Text("virustotal")
@@ -550,7 +689,8 @@ class TestView05:
         assert table.columns[0]._cells == [
             "Analysis:",
             "Reputation:",
-            "Last Modified:",
+            "Categories:",
+            "Updated:",
             "Last Seen:",
         ]
         assert table.columns[1].style == "none"
@@ -564,8 +704,29 @@ class TestView05:
                 ]
             ),
             Text("-1"),
-            "2022-08-17T05:30:23Z",
-            "2022-08-16T22:24:18Z",
+            Text(
+                (
+                    "ads/analytics, dynamic dns and isp sites, hosting, information technology, "
+                    "known infection source, mobile communications, not recommended site"
+                ),
+                spans=[
+                    Span(0, 13, "bright_white on black"),
+                    Span(13, 15, "default"),
+                    Span(15, 40, "bright_white on black"),
+                    Span(40, 42, "default"),
+                    Span(42, 49, "bright_white on black"),
+                    Span(49, 51, "default"),
+                    Span(51, 73, "bright_white on black"),
+                    Span(73, 75, "default"),
+                    Span(75, 97, "bright_white on black"),
+                    Span(97, 99, "default"),
+                    Span(99, 120, "bright_white on black"),
+                    Span(120, 122, "default"),
+                    Span(122, 142, "bright_white on black"),
+                ]
+            ),
+            display_timestamp("2022-08-17T05:30:23Z"),
+            display_timestamp("2022-08-16T22:24:18Z"),
         ]
 
 
@@ -576,11 +737,11 @@ class TestView06:
         assert whois.title == Text("whois")
 
         # Warning message
-        assert whois.renderable == Text("Unable to gather whois data")
+        assert whois.renderable == Text("No whois data found")
 
 
 class TestView07:
-    def test_resolutions_panel(self, view07):
+    def test_resolutions_panel(self, view07, theme, display_timestamp):
         res = view07.resolutions_panel()
         assert type(res) is Panel
 
@@ -590,7 +751,7 @@ class TestView07:
         # Heading
         assert group[0] == Text(
             "13.234.210.38",
-            spans=[Span(0, 13, "bold yellow link https://www.shodan.io/host/13.234.210.38")],
+            spans=[Span(0, 13, "bold yellow link https://virustotal.com/gui/ip-address/13.234.210.38")],
         )
 
         # Table
@@ -602,7 +763,10 @@ class TestView07:
             "ASN:",
             "ISP:",
             "Location:",
-            "Services:",
+            Text(
+                "Services:",
+                spans=[Span(0, 8, "link https://www.shodan.io/host/13.234.210.38")]
+            ),
             "Tags:",
             "Last Scan:",
         ]
@@ -610,8 +774,11 @@ class TestView07:
         assert group[1].columns[1].justify == "left"
         assert group[1].columns[1]._cells == [
             Text("0/94 malicious"),
-            "2022-08-06T14:56:20Z",
-            "16509 (Amazon Data Services India)",
+            display_timestamp("2022-08-06T14:56:20Z"),
+            Text(
+                "16509 (Amazon Data Services India)",
+                spans=[Span(7, 33, theme.asn_org)]
+            ),
             "Amazon.com, Inc.",
             Text(
                 "Mumbai, India",
@@ -641,7 +808,7 @@ class TestView07:
                     Span(0, 5, 'bright_white on black')
                 ]
             ),
-            "2022-08-21T07:21:05Z"
+            display_timestamp("2022-08-21T07:21:05Z")
         ]
 
         # Spacing
@@ -653,7 +820,7 @@ class TestView07:
         # Heading
         assert group[0] == Text(
             "192.30.255.113",
-            spans=[Span(0, 14, "bold yellow link https://www.shodan.io/host/192.30.255.113")],
+            spans=[Span(0, 14, "bold yellow link https://virustotal.com/gui/ip-address/192.30.255.113")],
         )
 
         # Table
@@ -665,15 +832,21 @@ class TestView07:
             "ASN:",
             "ISP:",
             "Location:",
-            "Services:",
+            Text(
+                "Services:",
+                spans=[Span(0, 8, "link https://www.shodan.io/host/192.30.255.113")]
+            ),
             "Last Scan:",
         ]
         assert group[1].columns[1].style == "none"
         assert group[1].columns[1].justify == "left"
         assert group[1].columns[1]._cells == [
             Text("1/94 malicious"),
-            "2022-06-21T18:10:54Z",
-            "36459 (GitHub, Inc.)",
+            display_timestamp("2022-06-21T18:10:54Z"),
+            Text(
+                "36459 (GitHub, Inc.)",
+                spans=[Span(7, 19, theme.asn_org)]
+            ),
             "GitHub, Inc.",
             Text(
                 "Seattle, United States",
@@ -697,7 +870,7 @@ class TestView07:
                     Span(19, 23, "cyan")
                 ]
             ),
-            "2022-08-21T22:33:53Z"
+            display_timestamp("2022-08-21T22:33:53Z")
         ]
 
         # Spacing
@@ -709,7 +882,7 @@ class TestView07:
         # Heading
         assert group[0] == Text(
             "13.234.176.102",
-            spans=[Span(0, 14, "bold yellow link https://www.shodan.io/host/13.234.176.102")],
+            spans=[Span(0, 14, "bold yellow link https://virustotal.com/gui/ip-address/13.234.176.102")],
         )
 
         # Unlike the previous entries, the table is inside a group of (Table, Text) due to
@@ -724,7 +897,10 @@ class TestView07:
             "ASN:",
             "ISP:",
             "Location:",
-            "Services:",
+            Text(
+                "Services:",
+                spans=[Span(0, 8, "link https://www.shodan.io/host/13.234.176.102")]
+            ),
             "Tags:",
             "Last Scan:",
         ]
@@ -732,8 +908,11 @@ class TestView07:
         assert table.columns[1].justify == "left"
         assert table.columns[1]._cells == [
             Text("0/94 malicious"),
-            "2015-08-17T07:11:53Z",
-            "16509 (Amazon Data Services India)",
+            display_timestamp("2015-08-17T07:11:53Z"),
+            Text(
+                "16509 (Amazon Data Services India)",
+                spans=[Span(7, 33, theme.asn_org)]
+            ),
             "Amazon.com, Inc.",
             Text(
                 "Mumbai, India",
@@ -763,7 +942,7 @@ class TestView07:
                     Span(0, 5, 'bright_white on black')
                 ]
             ),
-            "2022-08-21T02:13:35Z"
+            display_timestamp("2022-08-21T02:13:35Z")
         ]
 
         # Old timestamp warning
@@ -774,7 +953,7 @@ class TestView07:
 
 
 class TestView08:
-    def test_resolutions_panel(self, view08):
+    def test_resolutions_panel(self, view08, theme, display_timestamp):
         res = view08.resolutions_panel()
         assert type(res) is Panel
 
@@ -784,7 +963,7 @@ class TestView08:
         # Heading
         assert group[0] == Text(
             "199.232.34.194",
-            spans=[Span(0, 14, "bold yellow link https://www.shodan.io/host/199.232.34.194")],
+            spans=[Span(0, 14, "bold yellow link https://virustotal.com/gui/ip-address/199.232.34.194")],
         )
 
         # Table
@@ -796,7 +975,10 @@ class TestView08:
             "ASN:",
             "ISP:",
             "Location:",
-            "Services:",
+            Text(
+                "Services:",
+                spans=[Span(0, 8, "link https://www.shodan.io/host/199.232.34.194")]
+            ),
             "Tags:",
             "Last Scan:",
         ]
@@ -804,8 +986,11 @@ class TestView08:
         assert group[1].columns[1].justify == "left"
         assert group[1].columns[1]._cells == [
             Text("0/93 malicious"),
-            "2022-06-03T22:32:19Z",
-            "54113 (Fastly, Inc.)",
+            display_timestamp("2022-06-03T22:32:19Z"),
+            Text(
+                "54113 (Fastly, Inc.)",
+                spans=[Span(7, 19, theme.asn_org)]
+            ),
             "Fastly, Inc.",
             Text(
                 "Atlanta, United States",
@@ -834,7 +1019,7 @@ class TestView08:
                     Span(0, 3, 'bright_white on black'),
                 ]
             ),
-            "2022-08-21T01:33:13Z"
+            display_timestamp("2022-08-21T01:33:13Z")
         ]
 
         # Spacing
@@ -842,7 +1027,7 @@ class TestView08:
 
 
 class TestView09:
-    def test_resolutions_panel(self, view09):
+    def test_resolutions_panel(self, view09, theme, display_timestamp):
         res = view09.resolutions_panel()
         assert type(res) is Panel
 
@@ -852,7 +1037,7 @@ class TestView09:
         # Heading
         assert group[0] == Text(
             "1.0.0.1",
-            spans=[Span(0, 7, "bold yellow link https://www.shodan.io/host/1.0.0.1")],
+            spans=[Span(0, 7, "bold yellow link https://virustotal.com/gui/ip-address/1.0.0.1")],
         )
 
         # Table
@@ -866,15 +1051,21 @@ class TestView09:
             "ASN:",
             "ISP:",
             "Location:",
-            "Services:",
+            Text(
+                "Services:",
+                spans=[Span(0, 8, "link https://www.shodan.io/host/1.0.0.1")]
+            ),
             "Last Scan:",
         ]
         assert table.columns[1].style == "none"
         assert table.columns[1].justify == "left"
         assert table.columns[1]._cells == [
             Text("2/94 malicious"),
-            "2020-08-01T22:07:20Z",
-            "13335 (APNIC and Cloudflare DNS Resolver project)",
+            display_timestamp("2020-08-01T22:07:20Z"),
+            Text(
+                "13335 (APNIC and Cloudflare DNS Resolver project)",
+                spans=[Span(7, 48, theme.asn_org)],
+            ),
             "Cloudflare, Inc.",
             Text(
                 "Los Angeles, United States",
@@ -928,7 +1119,7 @@ class TestView09:
                     Span(96, 100, "cyan"),
                 ]
             ),
-            "2022-08-22T02:35:34Z"
+            display_timestamp("2022-08-22T02:35:34Z")
         ]
 
         # Old timestamp warning
@@ -936,3 +1127,146 @@ class TestView09:
 
         # Spacing
         assert res.renderable.renderables[1] == Text("\n+1 more")
+
+
+class TestView10:
+    def test_whois_panel(self, view10):
+        whois = view10.whois_panel()
+        assert type(whois) is Panel
+        assert whois.title == Text("whois")
+
+        # Warning message
+        assert whois.renderable == Text("No whois data found")
+
+
+class TestView11:
+    def test_resolutions_panel(self, view11, theme, display_timestamp):
+        res = view11.resolutions_panel()
+        assert type(res) is Panel
+
+        # Entry 1
+        group = res.renderable.renderables[0].renderables
+
+        # Heading
+        assert group[0] == Text(
+            "13.234.210.38",
+            spans=[Span(0, 13, "bold yellow link https://virustotal.com/gui/ip-address/13.234.210.38")],
+        )
+
+        # Table
+        assert group[1].columns[0].style == "bold bright_magenta"
+        assert group[1].columns[0].justify == "left"
+        assert group[1].columns[0]._cells == [
+            "Analysis:",
+            "Resolved:",
+            "ASN:",
+            "ISP:",
+            "Location:",
+            "Tags:",
+            "Last Scan:",
+        ]
+        assert group[1].columns[1].style == "none"
+        assert group[1].columns[1].justify == "left"
+        assert group[1].columns[1]._cells == [
+            Text("0/94 malicious"),
+            display_timestamp("2022-08-06T14:56:20Z"),
+            Text(
+                "16509 (Amazon Data Services India)",
+                spans=[Span(7, 33, theme.asn_org)]
+            ),
+            "Amazon.com, Inc.",
+            Text(
+                "Mumbai, India",
+                spans=[
+                    Span(6, 8, "default"),
+                ]
+            ),
+            Text(
+                "cloud",
+                spans=[
+                    Span(0, 5, 'bright_white on black')
+                ]
+            ),
+            display_timestamp("2022-08-21T07:21:05Z")
+        ]
+
+
+class TestView12:
+    def test_whois_panel(self, view12):
+        whois = view12.whois_panel()
+        assert type(whois) is Panel
+        assert whois.title == Text("whois")
+
+        # Heading
+        assert whois.renderable.renderables[0] == Text(
+            "hotmail.com",
+            spans=[Span(0, 11, "bold yellow")]
+        )
+
+        # Table
+        table = whois.renderable.renderables[1]
+        assert type(table) is Table
+        assert table.columns[0].style == "bold bright_magenta"
+        assert table.columns[0].justify == "left"
+        assert table.columns[0]._cells == [
+            "Registrar:",
+            "Organization:",
+            "Name:",
+            "Email:",
+            "Phone:",
+            "Street:",
+            "City:",
+            "State:",
+            "Country:",
+            "Postcode:",
+            "Nameservers:",
+            "Registered:",
+            "Updated:",
+            "Expires:",
+        ]
+        assert table.columns[1].style == "none"
+        assert table.columns[1].justify == "left"
+        assert [str(c) for c in table.columns[1]._cells] == [  # Just test the strings (no style)
+            "MarkMonitor, Inc.",
+            "Microsoft Corporation",
+            "Domain Administrator",
+            "domains@microsoft.com",
+            "+1.4258828080",
+            "One Microsoft Way,",
+            "Redmond",
+            "WA",
+            "US",
+            "98052",
+            ("ns4-205.azure-dns.info, ns3-205.azure-dns.org, ns1-205.azure-dns.com, "
+             "ns2-205.azure-dns.net"),
+            "1996-03-27T05:00:00Z",
+            "2021-02-02T17:08:19Z",
+            "2024-03-27T07:00:00Z",
+        ]
+
+
+class TestView13:
+    def test_whois_panel(self, view13):
+        whois = view13.whois_panel()
+        assert type(whois) is Panel
+        assert whois.title == Text("whois")
+
+        # No Heading!
+
+        # Table
+        table = whois.renderable
+        assert type(table) is Table
+        assert table.columns[0].style == "bold bright_magenta"
+        assert table.columns[0].justify == "left"
+        assert table.columns[0]._cells == [
+            "Registered:",
+            "Updated:",
+            "Expires:",
+        ]
+        assert table.columns[1].style == "none"
+        assert table.columns[1].justify == "left"
+        assert [str(c) for c in table.columns[1]._cells] == [  # Just test the strings (no style)
+            "1996-08-01T00:00:00Z",
+            "2020-12-10T00:00:00Z",
+            "2025-12-13T00:00:00Z",
+        ]
