@@ -1,3 +1,4 @@
+import abc
 import json
 import requests
 
@@ -5,11 +6,11 @@ from typing import Optional, Union
 
 
 class AbstractAttribute:
-    def __get__(self, obj, type):
+    def __get__(self, obj, type):  # pragma: no coverage
         raise NotImplementedError("This attribute must be set")
 
 
-class BaseClient:
+class BaseClient(abc.ABC):
     """
     Base client
     """
@@ -18,6 +19,11 @@ class BaseClient:
     def __init__(self) -> None:
         self.s = requests.Session()
 
+    @property
+    @abc.abstractmethod
+    def name(self) -> str:  # pragma: no coverage
+        return NotImplemented
+
     def _get(self, request: str, params: Optional[dict] = None) -> dict:
         resp = self.s.get(self.baseurl + request, params=params)
         resp.raise_for_status()
